#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-07-29
#
# 2016-07-29:  monitor Check Point FWM log server status
# 2018-05-08:  changed snmp scan and inventory function
# 2018-03-16:  added R80.10 MIBs
# 2018-05-30:  removed empty OIDs CHECKPOINT-MIB::lsConnectedClientsEntry,
#                                 CHECKPOINT-MIB::lsConnectedGatewaysEntry
#                                 CHECKPOINT-MIB::lsLoggingInfo
# 2020-06-08: changed snmp-scan function
# 2021-06-14: rewrite for cmk 2.0
# 2021-08-09: fixed empty string_table
# 2021-08-16: code cleanup
#

#
# sample snmpwalk (R77.30)
#
# .1.3.6.1.4.1.2620.1.11.1.0 = STRING: "Check Point Log Server"
# .1.3.6.1.4.1.2620.1.11.2.0 = INTEGER: 6
# .1.3.6.1.4.1.2620.1.11.3.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.11.4.0 = INTEGER: 4663
# .1.3.6.1.4.1.2620.1.11.5.0 = INTEGER: 1
# .1.3.6.1.4.1.2620.1.11.101.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.11.102.0 = STRING: "OK"
# .1.3.6.1.4.1.2620.1.11.103.0 = STRING: "OK"
#
# CHECKPOINT-MIB::lsProdName.0 = STRING: Check Point Log Server
# CHECKPOINT-MIB::lsVerMajor.0 = INTEGER: 6
# CHECKPOINT-MIB::lsVerMinor.0 = INTEGER: 0
# CHECKPOINT-MIB::lsBuildNumber.0 = INTEGER: 4663
# CHECKPOINT-MIB::lsFwmIsAlive.0 = INTEGER: 1
# CHECKPOINT-MIB::lsStatCode.0 = INTEGER: 0
# CHECKPOINT-MIB::lsStatShortDescr.0 = STRING: OK
# CHECKPOINT-MIB::lsStatLongDescr.0 = STRING: OK
#
# sample snmpwalk (R80.10, SmartEvent Server)
#
# .1.3.6.1.4.1.2620.1.11.1.0 = STRING: "Check Point Log Server"
# .1.3.6.1.4.1.2620.1.11.2.0 = INTEGER: 6
# .1.3.6.1.4.1.2620.1.11.3.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.11.4.0 = INTEGER: 39081
# .1.3.6.1.4.1.2620.1.11.5.0 = INTEGER: 1
# .1.3.6.1.4.1.2620.1.11.14.1.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.11.14.2.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.11.14.3.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.11.14.4.1.1.1.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.11.14.4.1.2.1.0 = STRING: "Local Clients"
# .1.3.6.1.4.1.2620.1.11.14.4.1.3.1.0 = STRING: "Connected"
# .1.3.6.1.4.1.2620.1.11.14.4.1.4.1.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.11.14.4.1.5.1.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.11.14.5.1.0 = STRING: "20171"
# .1.3.6.1.4.1.2620.1.11.14.5.2.0 = STRING: "2607010"
# .1.3.6.1.4.1.2620.1.11.14.5.3.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.11.14.5.4.0 = STRING: "9021"
# .1.3.6.1.4.1.2620.1.11.14.5.5.0 = Gauge32: 4
# .1.3.6.1.4.1.2620.1.11.14.5.6.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.11.14.5.7.0 = Gauge32: 10
# .1.3.6.1.4.1.2620.1.11.14.5.8.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.11.14.5.9.0 = Gauge32: 10
# .1.3.6.1.4.1.2620.1.11.14.5.10.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.11.14.5.11.0 = Gauge32: 27275
# .1.3.6.1.4.1.2620.1.11.14.5.12.0 = Gauge32: 1001
# .1.3.6.1.4.1.2620.1.11.14.5.13.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.11.14.6.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.11.101.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.11.102.0 = STRING: "OK"
# .1.3.6.1.4.1.2620.1.11.103.0 = STRING: "OK"
#
# CHECKPOINT-MIB::lsProdName.0 = STRING: Check Point Log Server
# CHECKPOINT-MIB::lsVerMajor.0 = INTEGER: 6
# CHECKPOINT-MIB::lsVerMinor.0 = INTEGER: 0
# CHECKPOINT-MIB::lsBuildNumber.0 = INTEGER: 39081
# CHECKPOINT-MIB::lsFwmIsAlive.0 = INTEGER: 1
# CHECKPOINT-MIB::lsLogReceiveRate.0 = Gauge32: 0
# CHECKPOINT-MIB::lsLogReceiveRatePeak.0 = Gauge32: 1
# CHECKPOINT-MIB::lsLogReceiveRate10Min.0 = Gauge32: 0
# CHECKPOINT-MIB::lsGWIndex.1.0 = Gauge32: 1
# CHECKPOINT-MIB::lsGWIP.1.0 = STRING: Local Clients
# CHECKPOINT-MIB::lsGWState.1.0 = STRING: Connected
# CHECKPOINT-MIB::lsGWLastLoginTime.1.0 = STRING: N/A
# CHECKPOINT-MIB::lsGWLogReceiveRate.1.0 = Gauge32: 0
# CHECKPOINT-MIB::lsIndexerInfoTotalReadLogs.0 = Wrong Type (should be Gauge32 or Unsigned32): STRING: "20171"
# CHECKPOINT-MIB::lsIndexerInfoTotalUpdatesAndLogsIndexed.0 = Wrong Type (should be Gauge32 or Unsigned32): STRING: "2607010"
# CHECKPOINT-MIB::lsIndexerInfoTotalReadLogsErrors.0 = Wrong Type (should be Gauge32 or Unsigned32): STRING: "0"
# CHECKPOINT-MIB::lsIndexerInfoTotalUpdatesAndLogsIndexedErrors.0 = Wrong Type (should be Gauge32 or Unsigned32): STRING: "9021"
# CHECKPOINT-MIB::lsIndexerInfoUpdatesAndLogsIndexedRate.0 = Gauge32: 4
# CHECKPOINT-MIB::lsIndexerInfoReadLogsRate.0 = Gauge32: 0
# CHECKPOINT-MIB::lsIndexerInfoUpdatesAndLogsIndexedRatePeak.0 = Gauge32: 10
# CHECKPOINT-MIB::lsIndexerInfoReadLogsRatePeak.0 = Gauge32: 1
# CHECKPOINT-MIB::lsIndexerInfo.9.0 = Gauge32: 10
# CHECKPOINT-MIB::lsIndexerInfo.10.0 = Gauge32: 1
# CHECKPOINT-MIB::lsIndexerInfo.11.0 = Gauge32: 27275
# CHECKPOINT-MIB::lsIndexerInfo.12.0 = Gauge32: 1001
# CHECKPOINT-MIB::lsIndexerInfo.13.0 = Gauge32: 0
# CHECKPOINT-MIB::lsLogReceiveRate1Hour.0 = Gauge32: 0
# CHECKPOINT-MIB::lsStatCode.0 = INTEGER: 0
# CHECKPOINT-MIB::lsStatShortDescr.0 = STRING: OK
# CHECKPOINT-MIB::lsStatLongDescr.0 = STRING: OK
#
# R80.10 MD Logserver
# .1.3.6.1.4.1.2620.1.11.1.0 = STRING: "Check Point Log Server"
# .1.3.6.1.4.1.2620.1.11.2.0 = INTEGER: 6
# .1.3.6.1.4.1.2620.1.11.3.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.11.4.0 = INTEGER: 39081
# .1.3.6.1.4.1.2620.1.11.5.0 = INTEGER: 0
# .1.3.6.1.4.1.2620.1.11.14.1.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.11.14.2.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.11.14.3.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.11.14.4.1.1.1.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.11.14.4.1.2.1.0 = STRING: "Local Clients"
# .1.3.6.1.4.1.2620.1.11.14.4.1.3.1.0 = STRING: "Connected"
# .1.3.6.1.4.1.2620.1.11.14.4.1.4.1.0 = STRING: "N/A"
# .1.3.6.1.4.1.2620.1.11.14.4.1.5.1.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.11.14.5.1.0 = STRING: "94"
# .1.3.6.1.4.1.2620.1.11.14.5.2.0 = STRING: "94"
# .1.3.6.1.4.1.2620.1.11.14.5.3.0 = STRING: "0"
# .1.3.6.1.4.1.2620.1.11.14.5.4.0 = STRING: "53"
# .1.3.6.1.4.1.2620.1.11.14.5.5.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.11.14.5.6.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.11.14.5.7.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.11.14.5.8.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.11.14.5.9.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.11.14.5.10.0 = Gauge32: 1
# .1.3.6.1.4.1.2620.1.11.14.5.11.0 = Gauge32: 2
# .1.3.6.1.4.1.2620.1.11.14.5.12.0 = Gauge32: 2
# .1.3.6.1.4.1.2620.1.11.14.5.13.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.11.14.6.0 = Gauge32: 0
# .1.3.6.1.4.1.2620.1.11.101.0 = INTEGER: 2
# .1.3.6.1.4.1.2620.1.11.102.0 = STRING: "Problem"
# .1.3.6.1.4.1.2620.1.11.103.0 = STRING: "Log Server is not running"
#
# sample info
# [[[u'Check Point Log Server', u'6', u'0', u'4663', u'1', u'0', u'OK', u'OK']], []]
#
# no logserver active
# [[], []]
#

from typing import NamedTuple, List, Optional

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    equals,
    Result,
    State,
    Metric,
    SNMPTree,
    startswith,
    all_of,
    any_of,
)


class CheckPointFwmR77(NamedTuple):
    lsvermajor: str
    lsverminor: str
    lsbuildnumber: int
    lsfwmisalive: int
    lsstatcode: int
    lsstatshortdescr: str
    lsstatlongdescr: str


class CheckPointFwmR80(NamedTuple):
    UpdatesAndLogsIndexedRate: int
    ReadLogsRate: int
    UpdatesAndLogsIndexedRatePeak: int
    ReadLogsRatePeak: int


class CheckPointFwm(NamedTuple):
    R77: CheckPointFwmR77
    R80: Optional[CheckPointFwmR80] = None


def parse_checkpoint_fwm_ls(string_table: List[StringTable]) -> Optional[CheckPointFwm]:
    try:
        lsvermajor, lsverminor, lsbuildnumber, lsfwmisalive, lsstatcode, lsstatshortdescr, \
        lsstatlongdescr = string_table[0][0]
    except (IndexError, ValueError):
        return

    checkpointfwmr77 = CheckPointFwmR77(
        lsvermajor=lsvermajor,
        lsverminor=lsverminor,
        lsbuildnumber=lsbuildnumber,
        lsfwmisalive=int(lsfwmisalive),
        lsstatcode=int(lsstatcode),
        lsstatshortdescr=lsstatshortdescr,
        lsstatlongdescr=lsstatlongdescr,
    )

    try:
        UpdatesAndLogsIndexedRate, ReadLogsRate, UpdatesAndLogsIndexedRatePeak, ReadLogsRatePeak = string_table[1][0]
    except IndexError:
        return CheckPointFwm(
            R77=checkpointfwmr77,
            R80=None,
        )
    except ValueError:
        return CheckPointFwm(
            R77=checkpointfwmr77,
            R80=None,
        )

    checkpointfwmr80 = CheckPointFwmR80(
        UpdatesAndLogsIndexedRate=int(UpdatesAndLogsIndexedRate),
        ReadLogsRate=int(ReadLogsRate),
        UpdatesAndLogsIndexedRatePeak=int(UpdatesAndLogsIndexedRatePeak),
        ReadLogsRatePeak=int(ReadLogsRatePeak),
    )

    return CheckPointFwm(
        R77=checkpointfwmr77,
        R80=checkpointfwmr80,
    )


def discovery_checkpoint_fwm_ls(section: CheckPointFwm) -> DiscoveryResult:
    yield Service()


def check_checkpoint_fwm_ls(params, section: CheckPointFwm) -> CheckResult:
    yield Result(state=State.OK, summary=f'Version: {section.R77.lsvermajor}.{section.R77.lsverminor}')
    yield Result(state=State.OK, summary=f'Build: {section.R77.lsbuildnumber}')

    if not params['ignore_status_on_r80_10']:
        if section.R77.lsfwmisalive != 1:
            yield Result(state=State.CRIT, summary='Is not alive')
        if section.R77.lsstatcode != 0:
            yield Result(state=State.CRIT,
                         summary=f'Status: {section.R77.lsstatshortdescr}',
                         details=f'Status: {section.R77.lsstatlongdescr}')
    if section.R80 is not None:
        for key, value in [
            ('checkpoint_fwm_ls_updatesandlogsindexedrate', section.R80.UpdatesAndLogsIndexedRate),
            ('checkpoint_fwm_ls_updatesandlogsindexedratepeak', section.R80.UpdatesAndLogsIndexedRatePeak),
            ('checkpoint_fwm_ls_readlogsrate', section.R80.ReadLogsRate),
            ('checkpoint_fwm_ls_readlogsratepeak', section.R80.ReadLogsRatePeak),
        ]:
            if value is not None:
                yield Metric(name=key, value=value)


register.snmp_section(
    name='checkpoint_fwm_ls',
    parse_function=parse_checkpoint_fwm_ls,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.11',  # CHECKPOINT-MIB::ls
            oids=[
                '2',  # lsVerMajor
                '3',  # lsVerMinor
                '4',  # lsBuildNumber
                '5',  # lsFwmIsAlive
                '101',  # lsStatCode
                '102',  # lsStatShortDescr
                '103',  # lsStatLongDescr
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2620.1.11.14.5',  # CHECKPOINT-MIB::lsIndexerInfo
            oids=[
                '5',  # lsIndexerInfoUpdatesAndLogsIndexedRate
                '6',  # lsIndexerInfoReadLogsRate
                '7',  # lsIndexerInfoUpdatesAndLogsIndexedRatePeak
                '8',  # lsIndexerInfoReadLogsRatePeak
            ]
        )

    ],
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_fwm_ls',
    service_name='FWM Log Server',
    discovery_function=discovery_checkpoint_fwm_ls,
    check_function=check_checkpoint_fwm_ls,
    check_default_parameters={'ignore_status_on_r80_10': False, },
    check_ruleset_name='checkpoint_fwm_ls',
)
